//  Xerver Free Web Server
//  Copyright (C) 2002-2007 Omid Rouhani
//
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
//  #############################################################
//  ##  YOU CAN CONTACT THE AUTHOR (OMID ROUHANI) AT:          ##
//  ##  HTTP://WWW.JAVASCRIPT.NU/XERVER/                       ##
//  ##                                                         ##
//  ##  IF YOUR SOFTWARE IS NOT RELEASED UNDER THE             ##
//  ##  GNU GENERAL PUBLIC LICENSE (GPL),                      ##
//  ##  PLEASE DO NOT COPY ANYTHING FROM THIS SOURCE CODE!!!   ##
//  ##                                                         ##
//  ##  FOR FULL LICENSE, PLEASE READ "XERVER LICENSE".        ##
//  #############################################################



package common;
import webserver.*;
import ftp_server.*;
import java.io.*;




/**
 *
 * <B>About this class:</B>
 * <BR>
 * This is like <CODE>File</CODE> but with a new method called <CODE>copyFile</CODE>.
 *
 * @author <a href="http://www.JavaScript.nu/xerver/" TARGET="_top">Omid Rouhani</a>
 * @version 1.0
 */

final public class FileWithCopy extends File
{
	final static private boolean b_showErrors=false;


	public FileWithCopy(File parent, String child)
	{
		super(parent,child);
	}

	public FileWithCopy(String pathname)
	{
		super(pathname);
	}

	public FileWithCopy(String parent, String child)
	{
		super(parent, child);
	}


	public boolean copyTo(String filename)
	{
		return copyFile(this, new File(filename));
	}


	/**
	* Copy this file to a new location.
	* <BR>
	* If destination file already exists, overwrite it...
	*/
	static public boolean copyFile(File f_src, File f_dest)
	{
		BufferedOutputStream bos=null;
		BufferedInputStream bis=null;
		try
		{
			if (!f_dest.exists())
				f_dest.createNewFile();	//We must first create the file before we write to the input stream

			bis=new BufferedInputStream(new FileInputStream(f_src));
			bos=new BufferedOutputStream(new FileOutputStream(f_dest));

			StreamIO.writeStreamToStream(bis,bos);

			bos.flush();
			bis.close();
			bos.close();
			bis=null;
			bos=null;

			return true;
		}
		catch (Exception e)
		{
			try		//Make sure to close bis even if we get Exception...
			{
				if (bis!=null)
					bis.close();
			}
			catch (Exception ex)
			{
				bis=null;
			}

			try		//Make sure to close bos even if we get Exception...
			{
				if (bos!=null)
					bos.close();
			}
			catch (Exception ex)
			{
				bos=null;
			}
			return false;
		}
	}
}